package com.usky.fire.service;

import com.usky.fire.domain.DemGridMemderVice;
import com.usky.common.mybatis.core.CrudService;

import java.util.List;

/**
 * <p>
 * 网格员管理附属表 服务类
 * </p>
 *
 * @author JCB
 * @since 2022-11-15
 */
public interface DemGridMemderViceService extends CrudService<DemGridMemderVice> {

    List<DemGridMemderVice> gridMemderViceList(List<Integer> gridMemberIdList);

    void addGridMemderVice(DemGridMemderVice demGridMemderVice);

    void updateGridMemderVice(DemGridMemderVice demGridMemderVice);

    List<String> gridMemderViceLists(Integer gridMemberId);

    void  delGridMemderVice(Integer id);
}
